Typed

Derive Macro Typed 

#[derive(Typed)]
Generates a Typed implementation from fields.

Only supports structs and enums.

This registers the target as a new Lua type that can be used to generate documentation.

Structs

Assigned as a lua class with it’s registered fields, indexes, methods, functions and their meta variants.

Enums

Assigned as an alias to a union of classes where each class is a enum variant. This is to best represent rust’s use of enums as unions.